#ifndef PROXYREADER_H
#define PROXYREADER_H

#include <boost/signals2/signal.hpp>
#include <vector>
#include "readerWaitset.h"
#include "domaincontext.h"
namespace easydds
{

template <typename T> class ProxyReader
{
public:
    explicit ProxyReader(const DomainContextPtr &domain_ctx,const DataReaderConfig& rc)
    {
        readerWaitset_ =std::make_unique< ReaderWaitsetContext<T>>(domain_ctx,rc ) ;
    }


    virtual ~ProxyReader()
    {

        readerWaitset_->stopThread();
        readerWaitset_->joinThread();//wait until the thread finish
        std::cout << "Thread exited" << std::endl;
        for (auto &it :connections_)
            it.disconnect();
    }

    typedef typename  ReaderWaitsetContext<T>::ReceivedData_SlotType ReceivedData_SlotType;
    void connectReceivedData(const ReceivedData_SlotType &subscriber)
    {
        boost::signals2::connection conn;
        conn=readerWaitset_->connectReceivedData(subscriber);
        connections_.emplace_back(conn);
    }





private:

    typedef typename std::unique_ptr<ReaderWaitsetContext<T> > readerWaitsetContextPtr;
    readerWaitsetContextPtr readerWaitset_;//specific class
    std::vector< boost::signals2::connection> connections_;

};




template <typename T> class Listener
{
public:
    typedef typename  ReaderWaitsetContext<T>::ReceivedData_SlotType ReceivedData_SlotType;
    Listener(const DomainContextPtr &domain_ctx,const DataReaderConfig &rc )
    {
        subscriber_=std::make_unique<ProxyReader<T>>(domain_ctx,rc);

        auto theFunctor=boost::bind(&Listener::slotAvailableData,this,_1);
        subscriber_->connectReceivedData(theFunctor);
    }

    void addSlot(const ReceivedData_SlotType &functor)
    {
        subscriber_->connectReceivedData(functor);
    }

protected:
    virtual void onSlotAvailableData(const T &sample)=0;


private:
    // slot methods
    void slotAvailableData(const T &sample)
    {
        onSlotAvailableData(sample);
    }

    typedef std::unique_ptr< ProxyReader<T> >  ProxyReaderPtr;

    ProxyReaderPtr subscriber_;
};
}
#endif // PROXYREADER_H
