#ifndef DOMAINCONTEXT_H
#define DOMAINCONTEXT_H

#include <dds/dds.hpp>


#include "ConfigValues.h"

namespace easydds
{

using QosProviderPtr       =std::unique_ptr<dds::core::QosProvider>;
using DomainParticipantPtr =std::unique_ptr<dds::domain::DomainParticipant>;
using PublisherPtr         =std::unique_ptr<dds::pub::Publisher>;
using SuscriberPtr         =std::unique_ptr<dds::sub::Subscriber>;


class DomainContext
{
public:
    explicit DomainContext(const configPtr &cfg)
    {
        QoSProvider_      =std::make_unique<dds::core::QosProvider>(cfg->qosFileName(),cfg->defaultProfile());
        // Get the current QosProviderParams from the QoSProvider_

       rti::core::QosProviderParams params = (*QoSProvider_.get())->provider_params();
       // load profiles from USER_QOS_PROFILES.xml file
       params.ignore_user_profile(false);

       // Ignore any qos profiles from other sources
       params.ignore_resource_profile (true);
       params.ignore_environment_profile (true);

       // Setting the params automatically reloads the profiles
       (*QoSProvider_.get())->provider_params(params);


       auto ls =(*QoSProvider_.get())-> qos_profile_libraries();
       std::cout << "--Loaded libraries--" << std::endl;
       for (auto &o:ls)
       {
           std::cout << o  << std::endl;
       }
        domainParticipant_=std::make_unique<dds::domain::DomainParticipant>(cfg->domainId(),QoSProvider_->participant_qos());
        publisher_        =std::make_unique<dds::pub::Publisher> (domainParticipant());
        subscriber_       =std::make_unique<dds::sub::Subscriber>(domainParticipant());





    }

    inline const dds::pub::Publisher& publisher() const
    {
        return *publisher_.get();
    }

    inline const dds::sub::Subscriber& subscriber() const
    {
        return *subscriber_.get();
    }
    inline const dds::domain::DomainParticipant& domainParticipant() const
    {
        return *domainParticipant_.get();
    }
    inline const dds::core::QosProvider& QoSProvider() const
    {
        return *QoSProvider_.get();
    }
    inline const QosProviderPtr& getQosProvider() const
    {
        return QoSProvider_;
    }


private:

    QosProviderPtr QoSProvider_;
    DomainParticipantPtr domainParticipant_;
    PublisherPtr publisher_;
    SuscriberPtr subscriber_;

};

using DomainContextPtr = std::unique_ptr<DomainContext>;
}
#endif // DOMAINCONTEXT_H
