#ifndef READERWAITSET_H
#define READERWAITSET_H

#include "boost/signals2/signal.hpp"
#include <thread>
#include "domaincontext.h"
#include "entityconfig.h"


namespace easydds{
template<typename T>
class ReaderWaitsetContext  {
public:


    using TopicType=dds::topic::Topic<T>;
    using TopicPtr = std::unique_ptr<TopicType> ;

    using DataReaderType=dds::sub::DataReader<T>;
    using DataReaderPtr=std::unique_ptr<DataReaderType>;

    typedef boost::signals2::signal<void (const T &data)>  signalReceivedData_t;//signal used by the TrivialListener
    typedef typename  signalReceivedData_t::slot_type ReceivedData_SlotType;

    /* Connect a slot to the signal which will be emitted whenever new data it will come */
    boost::signals2::connection connectReceivedData(const ReceivedData_SlotType &subscriber)
    {
        return sigReceivedData_.connect(subscriber);
    }


    ReaderWaitsetContext(const DomainContextPtr &dc,const DataReaderConfig &rdCfg)
    {
        topic_      = std::make_unique<TopicType>(dc->domainParticipant(),rdCfg.topic);

        (*dc->getQosProvider())->default_library(rdCfg.library);

        dds::sub::qos::DataReaderQos drQoS=dc->getQosProvider()->datareader_qos(rdCfg.profile);
        dataReader_ = std::make_unique<DataReaderType>(dc->subscriber(),*topic_.get(),drQoS);


        //start the thread to get data/ status
        thread_=std::thread(&ReaderWaitsetContext::threadFunc,this);

    }




    void stopThread(){
        guardCondition_.trigger_value(true);
    }

    void joinThread()
    {
        thread_.join();
    }

private:

    void threadFunc () {
         bool bEndWhile=false;
        auto reader = *dataReader_.get();
        dds::sub::cond::ReadCondition read_condition(
                    reader,
                    dds::sub::status::DataState::any(),
                    [&reader,this]()
        {
            // Take all samples
            dds::sub::LoanedSamples<T> samples = reader.take();
            for (auto sample : samples){
                if (sample.info().valid()){
                    sigReceivedData_(sample);


                }
            }
        } // The LoanedSamples destructor returns the loan
        );


        guardCondition_.handler([&bEndWhile]()
        {
            bEndWhile=true;
        });
        waitset_+=read_condition;
        waitset_+=guardCondition_;

        while (false==bEndWhile)
        {
            waitset_.dispatch(dds::core::Duration::infinite()); // Wait infinite
        }
        std::cout << "bEndWhile is true" << std::endl;
    }

    TopicPtr topic_;
    DataReaderPtr dataReader_;
    //encapsulated thread variable
    std::thread thread_;

    dds::core::cond::GuardCondition  guardCondition_;
    dds::core::cond::WaitSet waitset_;


    //signals for received data & Matched status
    signalReceivedData_t sigReceivedData_;



};

}
#endif // READERWAITSET_H
